PLAT_MAC = 1
PLAT_WINDOWS = 3

ENC_ROMAN = 0
ENC_UNICODE_11 = 1

LANG_ENGLISH = 1033

MACSTYLE = {'Regular': 0, 'Bold': 1, 'Italic': 2, 'Bold Italic': 3}


# Removes spaces from a string.
def sanitize(string):
    return string.replace(" ", "")


# Produces a unique ID for a style.
def getUniqueStyleID(style):
    id = style["name"]
    if "subfamily" in style:
        id = f"{style['subfamily']}-{id}"

    return sanitize(id)


def getFullName(style):
    familyName = style.get("prefFamily")
    if familyName == None:
        familyName = style.get("family")

    subfamilyName = style.get("prefSubfamily")
    if subfamilyName == None:
        subfamilyName = style.get("subfamily")

    return f"{familyName} {subfamilyName}"


def getPostscriptName(style):
    familyName = style.get("prefFamily")
    if familyName == None:
        familyName = style.get("family")

    subfamilyName = style.get("prefSubfamily")
    if subfamilyName == None:
        subfamilyName = style.get("subfamily")

    familyName = familyName.replace(" ", "")
    subfamilyName = subfamilyName.replace(" ", "")

    return f"{familyName}-{subfamilyName}"


# Rewrites the name table with new metadata.
def updateNames(font, style):
    nameTable = font["name"]

    nameTable.names = []

    family = style.get("family")
    subfamily = style.get("subfamily")
    prefFamily = style.get("prefFamily")
    prefSubfamily = style.get("prefSubfamily")

    fullName = getFullName(style)
    postscriptName = getPostscriptName(style)

    nameTable.setName(family, 1, PLAT_MAC, ENC_ROMAN, 0)
    nameTable.setName(family, 1, PLAT_WINDOWS, ENC_UNICODE_11, LANG_ENGLISH)

    nameTable.setName(subfamily, 2, PLAT_MAC, ENC_ROMAN, 0)
    nameTable.setName(subfamily, 2, PLAT_WINDOWS, ENC_UNICODE_11, LANG_ENGLISH)

    nameTable.setName(fullName, 3, PLAT_MAC, ENC_ROMAN, 0)
    nameTable.setName(fullName, 3, PLAT_WINDOWS, ENC_UNICODE_11, LANG_ENGLISH)

    nameTable.setName(fullName, 4, PLAT_MAC, ENC_ROMAN, 0)
    nameTable.setName(fullName, 4, PLAT_WINDOWS, ENC_UNICODE_11, LANG_ENGLISH)

    nameTable.setName("Version 1.000", 5, PLAT_MAC, ENC_ROMAN, 0)
    nameTable.setName("Version 1.000", 5, PLAT_WINDOWS, ENC_UNICODE_11,
                      LANG_ENGLISH)

    nameTable.setName(postscriptName, 6, PLAT_MAC, ENC_ROMAN, 0)
    nameTable.setName(postscriptName, 6, PLAT_WINDOWS, ENC_UNICODE_11,
                      LANG_ENGLISH)

    if prefFamily is not None:
        nameTable.setName(prefFamily, 16, PLAT_WINDOWS, ENC_UNICODE_11,
                          LANG_ENGLISH)

    if prefSubfamily is not None:
        nameTable.setName(prefSubfamily, 17, PLAT_WINDOWS, ENC_UNICODE_11,
                          LANG_ENGLISH)


def makeSelection(bits, style):
    bits = bits ^ bits

    if style == 'Regular':
        bits |= 0b1000000
    else:
        bits &= ~0b1000000

    if style == 'Bold' or style == 'BoldItalic':
        bits |= 0b100000
    else:
        bits &= ~0b100000

    if style == 'Italic':
        bits |= 0b1
    else:
        bits &= ~0b1

    if not bits:
        bits = 0b1000000

    return bits


def getMacStyle(style):
    return MACSTYLE[style]


def dropVariationTables(font):
    for tag in 'STAT cvar fvar gvar'.split():
        if tag in font.keys():
            del font[tag]



import os

from fontTools.ttLib import TTFont
from fontTools.varLib.instancer import instantiateVariableFont as instantiateFont
from tqdm import tqdm

from .util import updateNames, makeSelection, getPostscriptName, getMacStyle, sanitize, dropVariationTables


# Generates and writes each defined instance.
def generateInstances(config, args):

    # Create the output path if it doesn't exist.
    if not os.path.exists(args.outputPath):
        os.makedirs(args.outputPath)

    for style in tqdm(config, ascii=True, leave=False):
        font = TTFont(args.source)

        # Instantiate the font and update the name table.
        instantiateFont(font, style["axes"], inplace=True, overlap=True)
        updateNames(font, style)

        family = style.get("prefFamily")
        if family == None:
            family = style.get("family")

        subfamily = style.get("subfamily")
        prefSubfamily = style.get("prefSubfamily")
        if prefSubfamily == None:
            prefSubfamily = subfamily

        prefSubfamily = prefSubfamily.replace(" ", "")

        # Perform additional table fixups.
        font["head"].macStyle = getMacStyle(subfamily)
        font["OS/2"].fsSelection = makeSelection(font["OS/2"].fsSelection,
                                                 subfamily)

        # Override weight if requested.
        weightOverride = style.get("weightOverride")
        if weightOverride != None:
            font["OS/2"].usWeightClass = weightOverride

        # Override width if requested.
        widthOverride = style.get("widthOverride")
        if widthOverride != None:
            font["OS/2"].usWidthClass = widthOverride

        dropVariationTables(font)

        ext = args.format if args.format is not None else "ttf"
        filename = getPostscriptName(style) + f".{ext}"
        outputPath = os.path.join(args.outputPath, filename)

        font.flavor = args.format
        font.save(outputPath)



#!/usr/bin/env python3

# VFIT - Variable Font Instancing Tool
# Copyright (c) 2020 Jon Palmisciano

import argparse
import json
import sys

from . import core


# Loads and validates the config from the given path.
def loadConfig(path):
    config = {}

    # Attempt to read and parse the config file.
    with open(path, "r") as cfg_file:
        config = json.load(cfg_file)

    # Regular subfamily should be the default if subfamily is not specified.
    for style in config:
        if style.get("subfamily") is None:
            style["subfamily"] = "Regular"

    return config


def main():
    from argparse import ArgumentParser

    parser = ArgumentParser(
        prog="vfit",
        description=
        "Generate backwards-compatible, static instances of variable fonts.")

    parser.add_argument("config", help="the metadata/style definition file")
    parser.add_argument("source", help="the font to generate instances of")

    parser.add_argument("-o",
                        dest="outputPath",
                        metavar="PATH",
                        default=".",
                        help="where to place output files")

    parser.add_argument("-f",
                        dest="format",
                        default=None,
                        choices=["woff", "woff2"],
                        help="which format to output as")

    args = parser.parse_args()

    try:
        cfg = loadConfig(args.config)
    except ValueError as error:
        print(f"error: failed to load config, check your JSON\n{error}")
        sys.exit(1)

    core.generateInstances(cfg, args)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vfit']

package_data = \
{'': ['*']}

install_requires = \
['brotli>=1.0.7,<2.0.0', 'fonttools>=4.13.0,<5.0.0', 'tqdm>=4.48.0,<5.0.0']

entry_points = \
{'console_scripts': ['vfit = vfit.app:main']}

setup_kwargs = {
    'name': 'vfit',
    'version': '2.0.0',
    'description': 'Generate backwards-compatible, static instances of variable fonts',
    'long_description': '<div align="center">\n  <img src="vfit-logo.png" width="300">\n</div>\n\n## About\n\nVFIT (Variable Font Instancing Tool) allows you to generate customized,\nbackwards-compatible, static instances of a variable font from a configuration\nfile. Additionally, VFIT can manipulate font metadata such as the family name,\nversion, or copyright information.\n\n## Installation\n\nVFIT is not currently available on the Python Package Index. You can install\nVFIT by downloading a pre-built wheel from the Releases section or by building\none yourself.\n\n``` sh\n# Skip this step if you\'re downloading a prebuilt wheel.\n$ git clone https://github.com/jonpalmisc/vfit.git && cd vfit\n$ poetry build && cd dist\n\n# Install VFIT from the wheel.\n$ pip install vfit-version-py3-none-any.whl\n```\n\n## Usage\n\nTo begin, you will need a variable font file to work with. Your first step will\nbe creating a configuration file. See `sample.json` for an example.\n\nNext, run VFIT and pass your configuration and variable font file as arguments:\n\n``` sh\n$ vfit config.yaml Input-VF.ttf\n```\n\nIf you would like to generate instances into a specific directory, you can use\nthe `-o` option. For more options, see `./vift.py --help`.\n\n## Contributing\n\nAll contributions are welcome. If you find a bug or have a request for a\nfeature, feel free to create a new issue (or even better, a pull request).\n\n## Credits\n\nSpecial thanks to [Viktor Rubenko](https://github.com/ViktorRubenko) for\nhelping me get exported fonts to work on Windows!\n\n## License\n\nCopyright &copy; 2020 Jon Palmisciano\n\nVFIT is available under the MIT License. See [LICENSE.txt](LICENSE.txt) for\nmore information.\n',
    'author': 'Jon Palmisciano',
    'author_email': 'jp@jonpalmisc.com',
    'maintainer': 'Jon Palmisciano',
    'maintainer_email': 'jp@jonpalmisc.com',
    'url': 'https://github.com/jonpalmisc/vfit',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
